package com.muyu.engine.controller;

import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.poi.ExcelUtil;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;
import com.muyu.common.security.annotation.RequiresPermissions;
import com.muyu.engine.domain.RuleEngine;
import com.muyu.engine.service.RuleEngineService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 规则引擎Controller
 *
 * @author Saisai
 * @date 2024-05-03
 */
@RestController
@RequestMapping("/engine")
public class RuleEngineController extends BaseController {
    @Autowired
    private RuleEngineService ruleEngineService;

    /**
     * 查询规则引擎列表
     */
    @RequiresPermissions("engine:engine:list")
    @GetMapping("/list")
    public Result<TableDataInfo<RuleEngine>> list(RuleEngine ruleEngine) {
        startPage();
        List<RuleEngine> list = ruleEngineService.selectRuleEngineList(ruleEngine);
        return getDataTable(list);
    }

    /**
     * 导出规则引擎列表
     */
    @RequiresPermissions("engine:engine:export")
    @Log(title = "规则引擎", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, RuleEngine ruleEngine) {
        List<RuleEngine> list = ruleEngineService.selectRuleEngineList(ruleEngine);
        ExcelUtil<RuleEngine> util = new ExcelUtil<RuleEngine>(RuleEngine.class);
        util.exportExcel(response, list, "规则引擎数据");
    }

    /**
     * 获取规则引擎详细信息
     */
    @RequiresPermissions("engine:engine:query")
    @GetMapping(value = "/{id}")
    public Result getInfo(@PathVariable("id") Long id) {
        return success(ruleEngineService.selectRuleEngineById(id));
    }

    /**
     * 新增规则引擎
     */
    @RequiresPermissions("engine:engine:add")
    @Log(title = "规则引擎", businessType = BusinessType.INSERT)
    @PostMapping
    public Result add(@RequestBody RuleEngine ruleEngine) {
        return toAjax(ruleEngineService.insertRuleEngine(ruleEngine));
    }

    /**
     * 修改规则引擎
     */
    @RequiresPermissions("engine:engine:edit")
    @Log(title = "规则引擎", businessType = BusinessType.UPDATE)
    @PutMapping
    public Result edit(@RequestBody RuleEngine ruleEngine) {
        return toAjax(ruleEngineService.updateRuleEngine(ruleEngine));
    }

    /**
     * 删除规则引擎
     */
    @RequiresPermissions("engine:engine:remove")
    @Log(title = "规则引擎", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public Result remove(@PathVariable Long[] ids) {
        return toAjax(ruleEngineService.deleteRuleEngineByIds(ids));
    }


}
