package com.muyu.engine.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;
import com.muyu.common.security.annotation.RequiresPermissions;
import com.muyu.engine.domain.RuleEngineVersion;
import com.muyu.engine.service.RuleEngineVersionService;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.poi.ExcelUtil;
import com.muyu.common.core.web.page.TableDataInfo;

/**
 * 规则版本Controller
 *
 * @author Saisai
 * @date 2024-05-06
 */
@RestController
@RequestMapping("/version")
public class RuleEngineVersionController extends BaseController
{
    @Autowired
    private RuleEngineVersionService ruleEngineVersionService;

    /**
     * 查询规则版本列表
     */
    @RequiresPermissions("engine:version:list")
    @GetMapping("/list/{ruleEngineId}")
    public Result<TableDataInfo<RuleEngineVersion>> list(@PathVariable Long ruleEngineId)
    {
        List<RuleEngineVersion> list = ruleEngineVersionService.selectRuleEngineVersionList(ruleEngineId);
        return getDataTable(list);
    }

    /**
     * 导出规则版本列表
     */
    @RequiresPermissions("engine:version:export")
    @Log(title = "规则版本", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, Long ruleEngineId)
    {
        List<RuleEngineVersion> list = ruleEngineVersionService.selectRuleEngineVersionList(ruleEngineId);
        ExcelUtil<RuleEngineVersion> util = new ExcelUtil<RuleEngineVersion>(RuleEngineVersion.class);
        util.exportExcel(response, list, "规则版本数据");
    }

    /**
     * 获取规则版本详细信息
     */
    @RequiresPermissions("engine:version:query")
    @GetMapping(value = "/{id}")
    public Result getInfo(@PathVariable("id") Long id)
    {
        return success(ruleEngineVersionService.selectRuleEngineVersionById(id));
    }

    /**
     * 新增规则版本
     */
    @RequiresPermissions("engine:version:add")
    @Log(title = "规则版本", businessType = BusinessType.INSERT)
    @PostMapping
    public Result add(@RequestBody RuleEngineVersion ruleEngineVersion)
    {
        return toAjax(ruleEngineVersionService.insertRuleEngineVersion(ruleEngineVersion));
    }

    /**
     * 修改规则版本
     */
    @RequiresPermissions("engine:version:edit")
    @Log(title = "规则版本", businessType = BusinessType.UPDATE)
    @PutMapping
    public Result edit(@RequestBody RuleEngineVersion ruleEngineVersion)
    {
        return toAjax(ruleEngineVersionService.updateRuleEngineVersion(ruleEngineVersion));
    }

    /**
     * 删除规则版本
     */
    @RequiresPermissions("engine:version:remove")
    @Log(title = "规则版本", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public Result remove(@PathVariable Long[] ids)
    {
        return toAjax(ruleEngineVersionService.deleteRuleEngineVersionByIds(ids));
    }
}
