package com.muyu.engine.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.utils.DateUtils;
import com.muyu.engine.domain.RuleEngine;
import com.muyu.engine.mapper.RuleEngineMapper;
import com.muyu.engine.service.RuleEngineService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 规则引擎Service业务层处理
 *
 * @author Saisai
 * @date 2024-05-03
 */
@Service
@Log4j2
public class RuleEngineServiceImpl extends ServiceImpl<RuleEngineMapper, RuleEngine> implements RuleEngineService {
    @Autowired
    private RuleEngineMapper ruleEngineMapper;

    /**
     * 查询规则引擎
     *
     * @param id 规则引擎主键
     * @return 规则引擎
     */
    @Override
    public RuleEngine selectRuleEngineById(Long id) {
        return ruleEngineMapper.selectRuleEngineById(id);
    }

    /**
     * 查询规则引擎列表
     *
     * @param ruleEngine 规则引擎
     * @return 规则引擎
     */
    @Override
    public List<RuleEngine> selectRuleEngineList(RuleEngine ruleEngine) {
        return ruleEngineMapper.selectRuleEngineList(ruleEngine);
    }

    /**
     * 新增规则引擎
     *
     * @param ruleEngine 规则引擎
     * @return 结果
     */
    @Override
    public int insertRuleEngine(RuleEngine ruleEngine) {
        ruleEngine.setCreateTime(DateUtils.getNowDate());
        ruleEngine.setEngineCode("engine_custom_" + ruleEngine.getCode());
        boolean save = this.save(ruleEngine);
        if (save) {
            return 1;
        }
        return 0;
    }

    /**
     * 修改规则引擎
     *
     * @param ruleEngine 规则引擎
     * @return 结果
     */
    @Override
    public int updateRuleEngine(RuleEngine ruleEngine) {
        ruleEngine.setUpdateTime(DateUtils.getNowDate());
        return ruleEngineMapper.updateRuleEngine(ruleEngine);
    }

    /**
     * 批量删除规则引擎
     *
     * @param ids 需要删除的规则引擎主键
     * @return 结果
     */
    @Override
    public int deleteRuleEngineByIds(Long[] ids) {
        return ruleEngineMapper.deleteRuleEngineByIds(ids);
    }

    /**
     * 删除规则引擎信息
     *
     * @param id 规则引擎主键
     * @return 结果
     */
    @Override
    public int deleteRuleEngineById(Long id) {
        return ruleEngineMapper.deleteRuleEngineById(id);
    }


}
