package com.muyu.engine.service.impl;

import java.io.*;
import java.util.List;

import com.alibaba.nacos.shaded.com.google.common.base.Supplier;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.utils.DateUtils;
import com.muyu.engine.domain.RuleEngine;
import com.muyu.engine.service.RuleEngineService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.muyu.engine.mapper.RuleEngineVersionMapper;
import com.muyu.engine.domain.RuleEngineVersion;
import com.muyu.engine.service.RuleEngineVersionService;

import javax.servlet.ServletException;

/**
 * 规则版本Service业务层处理
 *
 * @author Saisai
 * @date 2024-05-06
 */
@Service
@Log4j2
public class RuleEngineVersionServiceImpl extends ServiceImpl<RuleEngineVersionMapper,RuleEngineVersion> implements RuleEngineVersionService
{
    @Autowired
    private RuleEngineVersionMapper ruleEngineVersionMapper;
    @Autowired
    private RuleEngineService ruleEngineService;

    /**
     * 查询规则版本
     *
     * @param id 规则版本主键
     * @return 规则版本
     */
    @Override
    public RuleEngineVersion selectRuleEngineVersionById(Long id)
    {
        return ruleEngineVersionMapper.selectRuleEngineVersionById(id);
    }

    /**
     * 查询规则版本列表
     *
     * @param ruleEngineId 规则版本
     * @return 规则版本
     */
    @Override
    public List<RuleEngineVersion> selectRuleEngineVersionList(Long ruleEngineId)
    {
        return list(new LambdaQueryWrapper<RuleEngineVersion>(){{
            eq(RuleEngineVersion::getRuleEngineId,ruleEngineId);
        }});
    }

    /**
     * 新增规则版本
     *
     * @param ruleEngineVersion 规则版本
     * @return 结果
     */
    @Override
    public int insertRuleEngineVersion(RuleEngineVersion ruleEngineVersion)
    {
        ruleEngineVersion.setCreateTime(DateUtils.getNowDate());
        return ruleEngineVersionMapper.insertRuleEngineVersion(ruleEngineVersion);
    }

    /**
     * 修改规则版本
     *
     * @param ruleEngineVersion 规则版本
     * @return 结果
     */
    @Override
    public int updateRuleEngineVersion(RuleEngineVersion ruleEngineVersion)
    {
        ruleEngineVersion.setUpdateTime(DateUtils.getNowDate());
        return ruleEngineVersionMapper.updateRuleEngineVersion(ruleEngineVersion);
    }

    /**
     * 批量删除规则版本
     *
     * @param ids 需要删除的规则版本主键
     * @return 结果
     */
    @Override
    public int deleteRuleEngineVersionByIds(Long[] ids)
    {
        return ruleEngineVersionMapper.deleteRuleEngineVersionByIds(ids);
    }

    /**
     * 删除规则版本信息
     *
     * @param id 规则版本主键
     * @return 结果
     */
    @Override
    public int deleteRuleEngineVersionById(Long id)
    {
        return ruleEngineVersionMapper.deleteRuleEngineVersionById(id);
    }




//    @Override
    public boolean testEngine(Long id) throws Exception {
//        RuleEngine ruleEngine = ruleEngineService.getById(id);
//        if (ruleEngine.getIsActivate().contains("no")) throw new ServletException("未激活");
//        if (ruleEngine.getStatus().contains("1")) throw new ServletException("已停用");
//        Scope scope = this.getOne(new LambdaQueryWrapper<Scope>() {{
//            eq(Scope::getRuleEngineId, ruleEngine.getId());
//            eq(Scope::getValue, "taskContext");
//        }});
//        String code = scope.getCode();
//        String path = code.substring(code.indexOf("com"), code.indexOf(";")).replaceAll("/.", "/").trim();
//        String fileName = code.substring(code.indexOf("class") + 6, code.indexOf("{")).trim();
//        String name = path+"."+fileName;
//        String javaPackageName = name.replace(".", File.separator)+".java";
//        String javaAbsolutePath = "D:/ruoyi/FinallyTest/muyu-modules/muyu-ruleEngine/muyu-ruleEngine-service/src/main/java/" +javaPackageName;
//        String jarAbsolutePath = "D:/ruoyi/FinallyTest/muyu-modules/muyu-ruleEngine/muyu-ruleEngine-service/target/classes/com/muyu/engine/controller";
//        Process process = Runtime.getRuntime().exec("javac -classpath "+ jarAbsolutePath+ " " + javaAbsolutePath);
//        try {
//            int exitVal = process.waitFor();
//            System.out.println("Process exitValue: " + exitVal);
//            ClassLoader classLoader = ReflectionUtils.class.getClassLoader();
//            Class aClass = classLoader.loadClass(name);
//            Object o = aClass.newInstance();
//            Method[] methods = aClass.getMethods();
//            for (Method method : methods) {
//                method.invoke(o,null);
//            }
//        } catch (Exception e) {
//            log.error(e.getMessage());
//        }
        return true;
    }

    /**
     * 引擎流程并执行测试
     *
     * @param id
     * @return
     */
//    @Override
    public boolean init(Long id) throws ServletException {
        RuleEngine ruleEngine = ruleEngineService.getById(id);
        if (ruleEngine.getIsActivate().contains("no")) throw new ServletException("未激活");
        if (ruleEngine.getStatus().contains("1")) throw new ServletException("已停用");
        Supplier<Boolean> booleanSupplier = () -> {
            try {
                return testEngine(ruleEngine);
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        };
        return booleanSupplier.get();
    }

    private Boolean testEngine(RuleEngine ruleEngine) throws ServletException, IOException {
        System.out.println("初始化");
        System.out.println(ruleEngine);
        RuleEngineVersion ruleEngineVersion = this.getOne(new LambdaQueryWrapper<RuleEngineVersion>() {{
            eq(RuleEngineVersion::getRuleEngineId, ruleEngine.getId());
        }});
        String code = ruleEngineVersion.getCodeIng();
        String path = code.substring(code.indexOf("com"), code.indexOf(";")).replaceAll("\\.", "/").trim();
        String fileName = code.substring(code.indexOf("class") + 6, code.indexOf("{")).trim();
        //代码编译
        File file = new File("D:/ruoyi/FinallyTest/muyu-modules/muyu-ruleEngine/muyu-ruleEngine-service/src/main/java/" + path + "/" + fileName + ".java");
        if (!file.exists()) {
            //不存在创建
            file.createNewFile();
        }else {
            //存在测重新创建
            file.delete();
            file.createNewFile();
        }
        String[] split = code.split("/n");
        //OutputStreamWriter对象将字符转换为字节流，指定了文件输出流和编码方式。
        OutputStreamWriter outputStreamWriter =
                new OutputStreamWriter(new FileOutputStream(file, true), "UTF-8");
        //BufferedWriter对象，用于缓冲写入数据，提高写入效率。
        BufferedWriter bw = new BufferedWriter(outputStreamWriter);
        for (String str : split) {
            bw.write(str);
            bw.newLine();
        }
        bw.close();
        outputStreamWriter.close();
        return true;
    }
}
