package com.muyu.system.domain;

import com.muyu.common.core.web.domain.BaseEntity;
import com.muyu.common.core.xss.Xss;
import com.muyu.system.domain.req.AsNoticeUserReq;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.experimental.SuperBuilder;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.Size;
import java.util.Date;
import java.util.function.Supplier;

/**
 * 通知公告表 sys_notice
 *
 * @author muyu
 */
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@EqualsAndHashCode(callSuper = true)
public class SysNotice extends BaseEntity {
    private static final long serialVersionUID = 1L;

    /**
     * 公告ID
     */
    private Long noticeId;

    /**
     * 公告标题
     */
    private String noticeTitle;

    /**
     * 公告类型（1通知 2公告）
     */
    private String noticeType;

    /**
     * 公告内容
     */
    private String noticeContent;

    /**
     * 公告状态（0正常 1关闭）
     */
    private String status;

    public static SysNotice saveBuilder(AsNoticeUserReq req, Supplier<String> user) {
        return SysNotice.builder()
                .noticeContent(req.getNoticeContent())
                .noticeTitle(req.getNoticeTitle())
                .noticeType(req.getNoticeType())
                .status(req.getStatus())
                .createBy(user.get())
                .createTime(new Date())
                .build();

    }


    @Xss(message = "公告标题不能包含脚本字符")
    @NotBlank(message = "公告标题不能为空")
    @Size(min = 0, max = 50, message = "公告标题不能超过50个字符")
    public String getNoticeTitle () {
        return noticeTitle;
    }

}
