package com.muyu.etl.domain;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.muyu.common.core.annotation.Excel;
import com.muyu.common.core.web.domain.BaseEntity;
import com.muyu.etl.domain.req.BasicConfigQueryReq;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.SuperBuilder;

import java.util.Date;
import java.util.function.Supplier;


/**
 *对象 basic_config_info
 *
 * @author Saisai
 * @date 2024-04-20
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@SuperBuilder
public class BasicConfigInfo extends BaseEntity
{
    private static final long serialVersionUID = 1L;

    /** 主键 */
    @TableId(value = "id", type = IdType.AUTO)
    private Long id;

    /** 接入源名称 */
    @Excel(name = "接入源名称")
    private String dataResourceName;

    /** 数据来源系统名称 */
    @Excel(name = "数据来源系统名称")
    private String dataSourcesSystemName;

    /** 主机ip地址 */
    @Excel(name = "主机ip地址")
    private String host;

    /** 端口 */
    @Excel(name = "端口")
    private String port;

    /** 数据接入类型 */
    @Excel(name = "数据接入类型")
    private String databaseType;

    /** 数据库名称 */
    @Excel(name = "数据库名称")
    private String databaseName;

    /** 初始化连接数量 */
    @Excel(name = "初始化连接数量")
    private Long initLinkNum;

    /** 最大连接数量 */
    @Excel(name = "最大连接数量")
    private Long maxLinkNum;

    /** 最大等待时间 */
    @Excel(name = "最大等待时间")
    private Long maxWaitTime;

    /** 最大等待次数 */
    @Excel(name = "最大等待次数")
    private Long maxWaitTimes;

    @Excel(name ="连接参数")
    private String connectionParams;

    @Excel(name ="用户名")
    private String username;

    @Excel(name ="密码")
    private String password;

    private String createBy;
    private String updateBy;
    private Date createTime;
    private Date updateTime;

    /**
     * 创建存储对象
     * @param configQueryReq
     * @param username
     * @return
     */
    public static BasicConfigInfo saveBuilder(BasicConfigQueryReq configQueryReq, Supplier<String> username) {
        return BasicConfigInfo.builder()
                .id(configQueryReq.getId())
                .databaseName(configQueryReq.getDatabaseName())
                .databaseType(configQueryReq.getDatabaseType())
                .dataResourceName(configQueryReq.getDataResourceName())
                .dataSourcesSystemName(configQueryReq.getDataSourcesSystemName())
                .host(configQueryReq.getHost())
                .port(configQueryReq.getPort())
                .initLinkNum(configQueryReq.getInitLinkNum())
                .maxLinkNum(configQueryReq.getMaxLinkNum())
                .maxWaitTime(configQueryReq.getMaxWaitTime())
                .createBy(username.get())
                .createTime(new Date())
                .build();
    }

    /**
     * 修改对象
     * @param configQueryReq
     * @param username
     * @return
     */
    public static BasicConfigInfo editBuilder(BasicConfigQueryReq configQueryReq, Supplier<String> username) {
        return BasicConfigInfo.builder()
                .id(configQueryReq.getId())
                .databaseName(configQueryReq.getDatabaseName())
                .databaseType(configQueryReq.getDatabaseType())
                .dataResourceName(configQueryReq.getDataResourceName())
                .dataSourcesSystemName(configQueryReq.getDataSourcesSystemName())
                .host(configQueryReq.getHost())
                .port(configQueryReq.getPort())
                .initLinkNum(configQueryReq.getInitLinkNum())
                .maxLinkNum(configQueryReq.getMaxLinkNum())
                .maxWaitTime(configQueryReq.getMaxWaitTime())
                .updateBy(username.get())
                .updateTime(new Date())
                .build();
    }
}
