package com.muyu.etl.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;
import com.muyu.common.security.annotation.RequiresPermissions;
import com.muyu.etl.domain.AssetImpower;
import com.muyu.etl.service.AssetImpowerService;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.poi.ExcelUtil;
import com.muyu.common.core.web.page.TableDataInfo;

/**
 * 资产赋权Controller
 *
 * @author Saisai
 * @date 2024-04-28
 */
@RestController
@RequestMapping("/impower")
public class AssetImpowerController extends BaseController
{
    @Autowired
    private AssetImpowerService assetImpowerService;

    /**
     * 查询资产赋权列表
     */
    @RequiresPermissions("etl:impower:list")
    @GetMapping("/list")
    public Result<TableDataInfo<AssetImpower>> list(AssetImpower assetImpower)
    {
        startPage();
        List<AssetImpower> list = assetImpowerService.selectAssetImpowerList(assetImpower);
        return getDataTable(list);
    }

    /**
     * 导出资产赋权列表
     */
    @RequiresPermissions("etl:impower:export")
    @Log(title = "资产赋权", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, AssetImpower assetImpower)
    {
        List<AssetImpower> list = assetImpowerService.selectAssetImpowerList(assetImpower);
        ExcelUtil<AssetImpower> util = new ExcelUtil<AssetImpower>(AssetImpower.class);
        util.exportExcel(response, list, "资产赋权数据");
    }

    /**
     * 获取资产赋权详细信息
     */
    @RequiresPermissions("etl:impower:query")
    @GetMapping(value = "/{id}")
    public Result getInfo(@PathVariable("id") Long id)
    {
        return success(assetImpowerService.selectAssetImpowerById(id));
    }

    /**
     * 新增资产赋权
     */
    @RequiresPermissions("etl:impower:add")
    @Log(title = "资产赋权", businessType = BusinessType.INSERT)
    @PostMapping
    public Result add(@RequestBody AssetImpower assetImpower)
    {
        return toAjax(assetImpowerService.insertAssetImpower(assetImpower));
    }

    /**
     * 修改资产赋权
     */
    @RequiresPermissions("etl:impower:edit")
    @Log(title = "资产赋权", businessType = BusinessType.UPDATE)
    @PutMapping
    public Result edit(@RequestBody AssetImpower assetImpower)
    {
        return toAjax(assetImpowerService.updateAssetImpower(assetImpower));
    }

    /**
     * 删除资产赋权
     */
    @RequiresPermissions("etl:impower:remove")
    @Log(title = "资产赋权", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public Result remove(@PathVariable Long[] ids)
    {
        return toAjax(assetImpowerService.deleteAssetImpowerByIds(ids));
    }


    /**
     * 删除资产赋权
     */
    @RequiresPermissions("etl:impower:list")
    @Log(title = "资产赋权", businessType = BusinessType.DELETE)
	@DeleteMapping("getPowerByTableId/{tableId}")
    public Result getPowerByTableId(@PathVariable Long tableId)
    {
        return Result.success(assetImpowerService.getPowerByTableId(tableId));
    }



}
