package com.muyu.etl.service.impl;

import java.util.List;
import java.util.stream.Collectors;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.utils.DateUtils;
import com.muyu.etl.domain.DictInfo;
import com.muyu.etl.domain.resp.BasicDictResp;
import com.muyu.etl.domain.resp.TableInfoStructureResp;
import com.muyu.etl.service.DictInfoService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.muyu.etl.mapper.AssetDataDictMapper;
import com.muyu.etl.domain.AssetDataDict;
import com.muyu.etl.service.AssetDataDictService;

import javax.servlet.ServletException;

/**
 * 资产数据字典Service业务层处理
 *
 * @author Saisai
 * @date 2024-04-24
 */
@Service
@Log4j2
public class AssetDataDictServiceImpl extends ServiceImpl<AssetDataDictMapper, AssetDataDict> implements AssetDataDictService {
    @Autowired
    private AssetDataDictMapper assetDataDictMapper;
    @Autowired
    private DictInfoService dictInfoService;

    /**
     * 查询资产数据字典
     *
     * @param id 资产数据字典主键
     * @return 资产数据字典
     */
    @Override
    public AssetDataDict selectAssetDataDictById(Long id) {
        return assetDataDictMapper.selectAssetDataDictById(id);
    }

    /**
     * 查询资产数据字典列表
     *
     * @param assetDataDict 资产数据字典
     * @return 资产数据字典
     */
    @Override
    public List<AssetDataDict> selectAssetDataDictList(AssetDataDict assetDataDict) {
        return assetDataDictMapper.selectAssetDataDictList(assetDataDict);
    }

    /**
     * 新增资产数据字典
     *
     * @param assetDataDict 资产数据字典
     * @return 结果
     */
    @Override
    public boolean insertAssetDataDict(AssetDataDict assetDataDict) throws ServletException {
        assetDataDict.setCreateTime(DateUtils.getNowDate());
        AssetDataDict one = this.getOne(new LambdaQueryWrapper<AssetDataDict>(AssetDataDict.class) {{
            eq(AssetDataDict::getDictType, assetDataDict.getDictType());
            eq(AssetDataDict::getBasicId, assetDataDict.getBasicId());
        }});
        if (one!=null){
            throw new ServletException("该字典已存在");
        }
        return this.saveOrUpdate(assetDataDict, new LambdaUpdateWrapper<AssetDataDict>(AssetDataDict.class) {{
            eq(AssetDataDict::getDictType,assetDataDict.getDictType());
            eq(AssetDataDict::getBasicId,assetDataDict.getBasicId());
        }});
    }

    /**
     * 修改资产数据字典
     *
     * @param assetDataDict 资产数据字典
     * @return 结果
     */
    @Override
    public int updateAssetDataDict(AssetDataDict assetDataDict) {
        assetDataDict.setUpdateTime(DateUtils.getNowDate());
        return assetDataDictMapper.updateAssetDataDict(assetDataDict);
    }

    /**
     * 批量删除资产数据字典
     *
     * @param ids 需要删除的资产数据字典主键
     * @return 结果
     */
    @Override
    public int deleteAssetDataDictByIds(Long[] ids) {
        return assetDataDictMapper.deleteAssetDataDictByIds(ids);
    }

    /**
     * 删除资产数据字典信息
     *
     * @param id 资产数据字典主键
     * @return 结果
     */
    @Override
    public int deleteAssetDataDictById(Long id) {
        return assetDataDictMapper.deleteAssetDataDictById(id);
    }

    @Override
    public List<BasicDictResp> getDict(Long basicId) {
        List<AssetDataDict> list = this.list(new LambdaQueryWrapper<AssetDataDict>() {{
            eq(AssetDataDict::getBasicId, basicId);
            or(assetDataDictLambdaQueryWrapper -> assetDataDictLambdaQueryWrapper.eq(AssetDataDict::getId, 1L));
        }});
        List<BasicDictResp> collect = list.stream().map(assetDataDict -> {
            List<DictInfo> dictInfoList = dictInfoService.list(
                    new LambdaQueryWrapper<DictInfo>()
                            .eq(DictInfo::getDictId, assetDataDict.getId()
                            )
            );
            return BasicDictResp.builder()
                    .id(assetDataDict.getId())
                    .dictType(assetDataDict.getDictType())
                    .dictName(assetDataDict.getDictName())
                    .dictInfoList(dictInfoList)
                    .basicId(assetDataDict.getBasicId())
                    .build();
        }).collect(Collectors.toList());
        return collect;
    }


}
