package com.muyu.etl.service.impl;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.constant.SecurityConstants;
import com.muyu.common.core.utils.DateUtils;
import com.muyu.common.system.domain.SysDept;
import com.muyu.common.system.domain.SysUser;
import com.muyu.common.system.remote.RemoteUserService;
import com.muyu.etl.domain.AssetImpower;
import com.muyu.etl.domain.TableInfo;
import com.muyu.etl.mapper.AssetImpowerMapper;
import com.muyu.etl.service.AssetImpowerService;
import com.muyu.etl.service.TableInfoService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * 资产赋权Service业务层处理
 *
 * @author Saisai
 * @date 2024-04-28
 */
@Service
@Log4j2
public class AssetImpowerServiceImpl extends ServiceImpl<AssetImpowerMapper, AssetImpower> implements AssetImpowerService {
    @Autowired
    private AssetImpowerMapper assetImpowerMapper;
    @Autowired
    private TableInfoService tableInfoService;
    @Autowired
    private RemoteUserService remoteUserService;

    /**
     * 查询资产赋权
     *
     * @param id 资产赋权主键
     * @return 资产赋权
     */
    @Override
    public AssetImpower selectAssetImpowerById(Long id) {
        return assetImpowerMapper.selectAssetImpowerById(id);
    }

    /**
     * 查询资产赋权列表
     *
     * @param assetImpower 资产赋权
     * @return 资产赋权
     */
    @Override
    public List<AssetImpower> selectAssetImpowerList(AssetImpower assetImpower) {
        return assetImpowerMapper.selectAssetImpowerList(assetImpower);
    }

    /**
     * 新增资产赋权
     *
     * @param assetImpower 资产赋权
     * @return 结果
     */
    @Override
    public int insertAssetImpower(AssetImpower assetImpower) {
        List<AssetImpower> rows = new ArrayList<>();
        assetImpower.setCreateTime(DateUtils.getNowDate());
        //创建一个部门容器和用户容器
        List<SysDept> deptList = remoteUserService.list(new SysDept(), SecurityConstants.INNER).getData();
        System.out.println(remoteUserService.list(new SysDept(), SecurityConstants.INNER));
        //单表单个用户赋权
        if (assetImpower.getTableId() != null && assetImpower.getUserId() != null) {
            rows.add(assetImpower);
        }
        //单表多用户赋权
        if (assetImpower.getTableId() != null) {
            //根据id过滤部门
            SysDept dept = remoteUserService.getInfo(assetImpower.getDeptId(), SecurityConstants.INNER).getData();
            deptList.addAll(getChildSysDept(deptList, dept));
            List<AssetImpower> finalRows = rows;
            deptList.stream().distinct().forEach(sysDept -> {
                //该部门下的用户列表
                remoteUserService.list(new SysUser() {{
                            setDeptId(sysDept.getDeptId());
                        }}, SecurityConstants.INNER).getData()
                        .forEach(sysUser -> finalRows.add(AssetImpower.saveAssetImpower(sysDept.getDeptId(), sysUser.getUserId(), assetImpower)));
            });
            rows = rows.stream().distinct().toList();
        }
        //多表单用户赋权
        if (assetImpower.getUserId() != null) {
            List<TableInfo> tableInfoList = tableInfoService.selectTableInfoList(new TableInfo() {{
                setBasicId(assetImpower.getBasicId());
            }});
            List<AssetImpower> finalRows = rows;
            tableInfoList.forEach(tableInfo -> finalRows.add(AssetImpower.saveAssetImpower(
                    assetImpower.getDeptId(), assetImpower.getUserId(), new AssetImpower() {{
                        setBasicId(assetImpower.getBasicId());
                        setTableId(tableInfo.getId());
                    }})));
        }
        boolean b = this.saveBatch(rows);
        return 1;
    }

    //递归获取子级部门
    public List<SysDept> getChildSysDept(List<SysDept> deptList, SysDept sysDept) {
        List<SysDept> list = new ArrayList<>();
        List<SysDept> deptChildList = new ArrayList<>(deptList);
        list.addAll(deptChildList.stream()
                .distinct()
                .filter(dept -> dept.getParentId().equals(sysDept.getDeptId())).toList());
        if (!list.isEmpty()) {
            list.forEach(sysDeptChild -> deptChildList.addAll(this.getChildSysDept(deptChildList, sysDeptChild)));
        }
        return list;
    }

    /**
     * 修改资产赋权
     *
     * @param assetImpower 资产赋权
     * @return 结果
     */
    @Override
    public int updateAssetImpower(AssetImpower assetImpower) {
        assetImpower.setUpdateTime(DateUtils.getNowDate());
        return assetImpowerMapper.updateAssetImpower(assetImpower);
    }

    /**
     * 批量删除资产赋权
     *
     * @param ids 需要删除的资产赋权主键
     * @return 结果
     */
    @Override
    public int deleteAssetImpowerByIds(Long[] ids) {
        return assetImpowerMapper.deleteAssetImpowerByIds(ids);
    }

    /**
     * 删除资产赋权信息
     *
     * @param id 资产赋权主键
     * @return 结果
     */
    @Override
    public int deleteAssetImpowerById(Long id) {
        return assetImpowerMapper.deleteAssetImpowerById(id);
    }

    @Override
    public List<AssetImpower> getPowerByTableId(Long tableId) {
        return this.selectAssetImpowerList(new AssetImpower() {{
            setTableId(tableId);
        }});
    }
}
