package com.muyu.etl.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.etl.domain.BasicConfigInfo;
import com.muyu.etl.domain.Structure;
import com.muyu.etl.domain.TableInfo;
import com.muyu.etl.domain.resp.BasicTableInfoResp;
import com.muyu.etl.domain.resp.TableInfoStructureResp;
import com.muyu.etl.domain.resp.TableTreeResp;
import com.muyu.etl.mapper.BasicConfigInfoMapper;
import com.muyu.etl.service.BasicConfigInfoService;
import com.muyu.etl.service.StructureService;
import com.muyu.etl.service.TableInfoService;
import lombok.SneakyThrows;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.ServletException;
import java.sql.*;
import java.util.Date;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.stream.Collectors;

/**
 * Service业务层处理
 *
 * @author muyu
 * @date 2024-04-20
 */
@Service
@Log4j2
public class BasicConfigInfoServiceImpl extends ServiceImpl<BasicConfigInfoMapper, BasicConfigInfo> implements BasicConfigInfoService {
    @Autowired
    private BasicConfigInfoMapper basicConfigInfoMapper;

    @Autowired
    private StructureService structureService;

    @Autowired
    private TableInfoService tableInfoService;


    /**
     * 查询
     *
     * @param id 主键
     * @return
     */
    @Override
    public BasicConfigInfo selectBasicConfigInfoById(Long id) {
        return basicConfigInfoMapper.selectBasicConfigInfoById(id);
    }

    /**
     * 查询列表
     *
     * @param basicConfigInfo
     * @return
     */
    @Override
    public List<BasicConfigInfo> selectBasicConfigInfoList(BasicConfigInfo basicConfigInfo) {
        return basicConfigInfoMapper.selectBasicConfigInfoList(basicConfigInfo);
    }

    /**
     * 新增
     *
     * @param configQueryReq
     * @return 结果
     */
    @Override
    public int insertBasicConfigInfo(BasicConfigInfo configQueryReq) {
        return basicConfigInfoMapper.insertBasicConfigInfo(configQueryReq);
    }

    /**
     * 修改
     *
     * @param configQueryReq
     * @return 结果
     */
    @Override
    public int updateBasicConfigInfo(BasicConfigInfo configQueryReq) {
        return basicConfigInfoMapper.updateBasicConfigInfo(configQueryReq);
    }

    /**
     * 批量删除
     *
     * @param ids 需要删除的主键
     * @return 结果
     */
    @Override
    public int deleteBasicConfigInfoByIds(Long[] ids) {
        return basicConfigInfoMapper.deleteBasicConfigInfoByIds(ids);
    }

    /**
     * 删除信息
     *
     * @param id 主键
     * @return 结果
     */
    @Override
    public int deleteBasicConfigInfoById(Long id) {
        return basicConfigInfoMapper.deleteBasicConfigInfoById(id);
    }

    /**
     * 测试连接并同步
     *
     * @param basicConfigInfo
     * @return
     * @throws ServletException
     */
    @Override
    @Transactional
    public boolean connectionTest(BasicConfigInfo basicConfigInfo) throws ServletException {
        //定义下面需要的对象
        String host = basicConfigInfo.getHost();
        String port = basicConfigInfo.getPort();
        String databaseName = basicConfigInfo.getDatabaseName();
        String databaseType = basicConfigInfo.getDatabaseType();
        String url = "jdbc:" + databaseType + "://" + host + ":" + port + "/" + databaseName + "?" + basicConfigInfo.getConnectionParams();
        String user = basicConfigInfo.getUsername();
        String password = basicConfigInfo.getPassword();
        Connection conn = null;
        try {
            conn = DriverManager.getConnection(url, user, password);
            System.out.println("Connected to the MySQL server successfully.");
            //同步数据库信息
            //树级结构，库，表
            TableInfo tableInfoInsert = TableInfo.builder()
                    .basicId(basicConfigInfo.getId())
                    .parentId(0L)
                    .tableRemark("")
                    .center("Y")
                    .type("dataSource")
                    .tableName(basicConfigInfo.getDataSourcesSystemName() + "(" + databaseName + ")")
                    .createBy(SecurityUtils.getUsername())
                    .createTime(new Date())
                    .build();
            tableInfoService.saveOrUpdate(tableInfoInsert, new LambdaUpdateWrapper<TableInfo>(TableInfo.class) {{
                eq(TableInfo::getTableName, tableInfoInsert.getTableName());
                eq(TableInfo::getBasicId, basicConfigInfo.getId());
            }});
            //查询列表中的所有tableInfo   basic_id    不存在删除
            //通过查询或去当前tableinfo对象的id
            TableInfo tableInfo = tableInfoService.selectTableInfoByName(tableInfoInsert);
            DatabaseMetaData metaData = conn.getMetaData();
            ResultSet rs = metaData.getTables(databaseName,
                    null, "%", new String[]{"TABLE", "VIEW"});
            while (rs.next()) {
                //表名
                String tableName = rs.getString("TABLE_NAME");
                String tableRemark = rs.getString("REMARKS");
                Connection finalConn = conn;
                PreparedStatement ps = conn.prepareStatement("select * from " + tableName);
                ResultSet rset = ps.executeQuery();
                Long rowCount = 0L;
                while (rset.next()) {
                    rowCount++;
                }
                TableInfo build = TableInfo.builder()
                        .basicId(basicConfigInfo.getId())
                        .tableName(tableName)
                        //bug点，tableRemark为空，造成空指针异常
                        .tableRemark(tableRemark == null ? "" : tableRemark)
                        .parentId(tableInfo.getId())
                        .type("dataTable")
                        .center("Y")
                        .updateBy(SecurityUtils.getUsername())
                        .dataNum(rowCount)
                        .updateTime(new Date())
                        .build();
                tableInfoService.saveOrUpdate(build, new LambdaUpdateWrapper<>(TableInfo.class) {{
                    eq(TableInfo::getTableName, build.getTableName());
                    eq(TableInfo::getBasicId, basicConfigInfo.getId());
                }});
                TableInfo table = tableInfoService.selectTableInfoByName(build);
                //线程池
                ExecutorService threadPool = Executors.newCachedThreadPool();

                threadPool.submit(() -> {
                    try {
                        syncData(finalConn, databaseName, table);
                    } catch (SQLException e) {
                        try {
                            throw new ServletException("同步数据失败");
                        } catch (ServletException ex) {
                            throw new RuntimeException(ex.getMessage());
                        }
                    }
                });
                Runnable thread = new Runnable() {
                    @SneakyThrows
                    @Override
                    public void run() {
                        try {
                            //同步
                            syncData(finalConn, databaseName, table);
                        } catch (SQLException e) {
                            log.error(e.getMessage());
                            throw new ServletException("连接失败");
                        }
                    }
                };
                thread.run();
                ps.close();
            }
            conn.close();
        } catch (SQLException e) {
            log.error(e.getMessage());
            throw new ServletException("连接失败");
        }
        return true;
    }

    /**
     * 同步数据
     *
     * @param conn
     * @param databaseName
     * @param table
     * @throws SQLException
     */
    public void syncData(Connection conn, String databaseName, TableInfo table) throws SQLException {
        ExecutorService threadPool = Executors.newCachedThreadPool();
        PreparedStatement ps = conn.prepareStatement(
                "    SELECT  " +
                        "    COLUMN_NAME , " +
                        "    COLUMN_COMMENT  ," +
                        "    CASE WHEN COLUMN_KEY = 'PRI' THEN '是' ELSE '否' END ," +
                        "    CASE \n" +
                        "    WHEN DATA_TYPE = 'int' THEN 'Integer' " +
                        "    WHEN DATA_TYPE = 'bigint' THEN 'Long' " +
                        "    WHEN DATA_TYPE = 'varchar' THEN 'String' " +
                        "    WHEN DATA_TYPE = 'decimal' THEN 'BigDecimal' " +
                        "    WHEN DATA_TYPE = 'tinyint' AND COLUMN_TYPE = 'tinyint(1)' THEN 'Boolean'" +
                        "    ELSE DATA_TYPE -- 如果无法映射，则返回原始数据库类型  \n" +
                        "    END , " +
                        "    DATA_TYPE , -- 原始的数据库类型  \n" +
                        "    COLUMN_TYPE , -- 更详细的数据库类型，可能包含长度、精度等  \n" +
                        "    CHARACTER_MAXIMUM_LENGTH ,  \n" +
                        "    NUMERIC_SCALE ,  \n" +
                        "    IS_NULLABLE ,  \n" +
                        "    COLUMN_DEFAULT   \n" +
                        "FROM  INFORMATION_SCHEMA.COLUMNS  WHERE  \n" +
                        "TABLE_SCHEMA = '" + databaseName + "' -- 替换为你的数据库名称  \n" +
                        "AND TABLE_NAME = '" + table.getTableName() + "'");
//        "SELECT COLUMN_NAME,DATA_TYPE,IS_NULLABLE,COLUMN_KEY,COLUMN_DEFAULT,
//        COLUMN_COMMENT,CHARACTER_MAXIMUM_LENGTH,NUMERIC_PRECISION,
//        NUMERIC_SCALE FROM INFORMATION_SCHEMA.COLUMNS
//        WHERE TABLE_SCHEMA = '数据库名' AND TABLE_NAME = '表名'"
        ResultSet resultSet = ps.executeQuery();
        while (resultSet.next()) {
            String columnName = String.valueOf(resultSet.getString(1));
            String columnComment = String.valueOf(resultSet.getObject(2));
            String columnKey = String.valueOf(resultSet.getObject(3));
            String end = String.valueOf(resultSet.getObject(4));
            String dataType = String.valueOf(resultSet.getObject(5));
            String columnType = String.valueOf(resultSet.getObject(6));
            String characterMaximumLength = String.valueOf(resultSet.getInt(7));
            String NumericScale = String.valueOf(resultSet.getInt(8));
            String isNullable = String.valueOf(resultSet.getObject(9));
            String columnDefault = String.valueOf(resultSet.getObject(10));
            Structure build = Structure.builder()
                    .tableId(table.getId())
                    .columnName(String.valueOf(columnName))
                    .columnRemark(columnComment)
                    .isPrimary("是".equals(columnKey) ? "Y" : "N")
                    .javaType(end)
                    .columnType(dataType)
                    .columnType(columnType)
                    .columnLength(characterMaximumLength)
                    .columnDecimals(NumericScale)
                    .isNull("YES".equals(isNullable) ? "Y" : "N")
                    .defaultValue(columnDefault)
                    .build();
            threadPool.submit(() -> {
                structureService.saveOrUpdate(build, new LambdaUpdateWrapper<Structure>() {{
                    eq(Structure::getTableId, build.getTableId());
                    eq(Structure::getColumnName, build.getColumnName());
                    eq(Structure::getColumnRemark, build.getColumnRemark());
                }});
            });
        }
        threadPool.shutdown();
        ps.close();
    }

    /**
     * 获取响应体对象
     *
     * @return
     */
    @Override
    public List<TableTreeResp> getTableTree() {
        ExecutorService threadPool = Executors.newCachedThreadPool();
        List<TableTreeResp> tableTreeRespList = tableInfoService.selectTableInfoList(new TableInfo() {{
            setParentId(0L);
        }}).stream().map(tableInfo -> {
            BasicConfigInfo basicConfigInfo = this.selectBasicConfigInfoById(tableInfo.getBasicId());
            List<TableInfoStructureResp> tableInfoStructureRespList = tableInfoService.selectTableInfoList(new TableInfo() {{
                setParentId(tableInfo.getId());
            }}).stream().map(info -> {
                TableInfoStructureResp tableInfoStructureResp = null;
                Future<TableInfoStructureResp> submit = threadPool.submit(() -> {
                    return TableInfoStructureResp.builder()
                            .id(info.getId())
                            .tableName(info.getTableName())
                            .center(info.getCenter())
                            .tableRemark(info.getTableRemark())
                            .type(info.getType())
                            .dataNum(info.getDataNum())
                            .parentId(info.getParentId())
                            .databaseType(basicConfigInfo.getDatabaseType())
                            .structureList(null)
                            .basicId(info.getBasicId())
                            .build();
                });
                try {
                    tableInfoStructureResp = submit.get();
                } catch (Exception e) {
                    log.error(e.getMessage());
                    throw new RuntimeException(e);
                }
                return tableInfoStructureResp;
            }).collect(Collectors.toList());
            return TableTreeResp.builder()
                    .tableInfo(tableInfo)
                    .basicConfigInfo(basicConfigInfo)
                    .Children(tableInfoStructureRespList)
                    .build();
        }).collect(Collectors.toList());
        threadPool.shutdown();
        while (true) if (threadPool.isTerminated()) break;

        return tableTreeRespList;
    }

    @Override
    public TableInfoStructureResp getTableInfo(Long tableId) {
        TableInfo tableInfo = tableInfoService.selectTableInfoById(tableId);
        BasicConfigInfo basicConfigInfo = this.selectBasicConfigInfoById(tableInfo.getBasicId());
        List<Structure> structureList = structureService.list(new LambdaQueryWrapper<Structure>()
                .eq(Structure::getTableId, tableInfo.getId()));
        return TableInfoStructureResp.builder()
                .id(tableInfo.getId())
                .tableName(tableInfo.getTableName())
                .center(tableInfo.getCenter())
                .tableRemark(tableInfo.getTableRemark())
                .dataNum(tableInfo.getDataNum())
                .parentId(tableInfo.getParentId())
                .databaseType(basicConfigInfo.getDatabaseType())
                .structureList(structureList)
                .basicId(tableInfo.getBasicId())
                .build();
    }

    @Override
    public BasicTableInfoResp getBasicTableInfo(Long tableId) {
        TableInfo tableInfo = tableInfoService.selectTableInfoById(tableId);
        BasicConfigInfo basicConfigInfo = this.selectBasicConfigInfoById(tableInfo.getBasicId());
        String host = basicConfigInfo.getHost();
        String port = basicConfigInfo.getPort();
        String databaseName = basicConfigInfo.getDatabaseName();
        String databaseType = basicConfigInfo.getDatabaseType();
        String url = "jdbc:" + databaseType + "://" + host + ":" + port + "/" + databaseName + "?" + basicConfigInfo.getConnectionParams();
        String user = basicConfigInfo.getUsername();
        String password = basicConfigInfo.getPassword();
        Long totalNum = null;
        BasicTableInfoResp basicTableInfoResp = null;
        Connection conn = null;
        try {
            conn = DriverManager.getConnection(url, user, password);
            PreparedStatement ps = conn.prepareStatement("select Count(0) from " + tableInfo.getTableName());
            ResultSet resultSet = ps.executeQuery();
            while (resultSet.next()){
                String a = String.valueOf(resultSet.getObject(1));
                totalNum = resultSet.getLong(1);
            }
            basicTableInfoResp = BasicTableInfoResp.builder()
                    .dataResourceName(basicConfigInfo.getDataResourceName())
                    .dataSourcesSystemName(basicConfigInfo.getDataSourcesSystemName())
                    .databaseName(basicConfigInfo.getDatabaseName())
                    .tableName(tableInfo.getTableName())
                    .tableRemark(tableInfo.getTableRemark())
                    .id(tableId)
                    .basicId(basicConfigInfo.getId())
                    .totalNum(totalNum)
                    .build();
            ps.close();
            conn.close();
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        return basicTableInfoResp;
    }

    }
