package com.muyu.etl.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.utils.DateUtils;
import com.muyu.etl.domain.DictInfo;
import com.muyu.etl.mapper.DictInfoMapper;
import com.muyu.etl.service.DictInfoService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.ServletException;
import java.util.List;

/**
 * 字典详细内容Service业务层处理
 *
 * @author Saisai
 * @date 2024-04-24
 */
@Log4j2
@Service
public class DictInfoServiceImpl extends ServiceImpl<DictInfoMapper, DictInfo> implements DictInfoService {
    @Autowired
    private DictInfoMapper dictInfoMapper;

    /**
     * 查询字典详细内容
     *
     * @param id 字典详细内容主键
     * @return 字典详细内容
     */
    @Override
    public DictInfo selectDictInfoById(Long id) {
        return dictInfoMapper.selectDictInfoById(id);
    }

    /**
     * 查询字典详细内容列表
     *
     * @param dictInfo 字典详细内容
     * @return 字典详细内容
     */
    @Override
    public List<DictInfo> selectDictInfoList(DictInfo dictInfo) {
        return dictInfoMapper.selectDictInfoList(dictInfo);
    }

    /**
     * 新增字典详细内容
     *
     * @param dictInfo 字典详细内容
     * @return 结果
     */
    @Override
    public boolean insertDictInfo(DictInfo dictInfo) throws ServletException {
        dictInfo.setCreateTime(DateUtils.getNowDate());
        DictInfo one = this.getOne(new LambdaQueryWrapper<DictInfo>() {{
            eq(DictInfo::getDictId, dictInfo.getDictId());
            eq(DictInfo::getInfoName, dictInfo.getInfoName());
            eq(DictInfo::getInfoValue, dictInfo.getInfoValue());
        }});
        if (one!=null){
            throw new ServletException("该字典内容已存在");
        }
        boolean b = this.saveOrUpdate(dictInfo, new LambdaUpdateWrapper<DictInfo>(DictInfo.class) {{
            eq(DictInfo::getDictId, dictInfo.getDictId());
            eq(DictInfo::getInfoName, dictInfo.getInfoName());
            eq(DictInfo::getInfoValue, dictInfo.getInfoValue());
        }});
        return b;
    }

    /**
     * 修改字典详细内容
     *
     * @param dictInfo 字典详细内容
     * @return 结果
     */
    @Override
    public int updateDictInfo(DictInfo dictInfo) {
        dictInfo.setUpdateTime(DateUtils.getNowDate());
        return dictInfoMapper.updateDictInfo(dictInfo);
    }

    /**
     * 批量删除字典详细内容
     *
     * @param ids 需要删除的字典详细内容主键
     * @return 结果
     */
    @Override
    public int deleteDictInfoByIds(Long[] ids) {
        return dictInfoMapper.deleteDictInfoByIds(ids);
    }

    /**
     * 删除字典详细内容信息
     *
     * @param id 字典详细内容主键
     * @return 结果
     */
    @Override
    public int deleteDictInfoById(Long id) {
        return dictInfoMapper.deleteDictInfoById(id);
    }
}
