package com.muyu.etl.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.utils.DateUtils;
import com.muyu.etl.domain.Structure;
import com.muyu.etl.mapper.StructureMapper;
import com.muyu.etl.service.StructureService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 结构Service业务层处理
 *
 * @author Saisai
 * @date 2024-04-22
 */
@Service
@Log4j2
public class StructureServiceImpl extends ServiceImpl<StructureMapper, Structure> implements StructureService
{
    @Autowired
    private StructureMapper structureMapper;

    /**
     * 查询结构
     *
     * @param id 结构主键
     * @return 结构
     */
    @Override
    public Structure selectStructureById(Long id)
    {
        return structureMapper.selectStructureById(id);
    }

    /**
     * 查询结构列表
     *
     * @param structure 结构
     * @return 结构
     */
    @Override
    public List<Structure> selectStructureList(Structure structure)
    {
        return structureMapper.selectStructureList(structure);
    }

    /**
     * 新增结构
     *
     * @param structure 结构
     * @return 结果
     */
    @Override
    public int insertStructure(Structure structure)
    {
        structure.setCreateTime(DateUtils.getNowDate());
        return structureMapper.insertStructure(structure);
    }

    /**
     * 修改结构
     *
     * @param structure 结构
     * @return 结果
     */
    @Override
    public int updateStructure(Structure structure)
    {
        structure.setUpdateTime(DateUtils.getNowDate());
        return structureMapper.updateStructure(structure);
    }

    /**
     * 批量删除结构
     *
     * @param ids 需要删除的结构主键
     * @return 结果
     */
    @Override
    public int deleteStructureByIds(Long[] ids)
    {
        return structureMapper.deleteStructureByIds(ids);
    }

    /**
     * 删除结构信息
     *
     * @param id 结构主键
     * @return 结果
     */
    @Override
    public int deleteStructureById(Long id)
    {
        return structureMapper.deleteStructureById(id);
    }

    @Override
    public boolean updateStructureInfoDict(Structure structure) {
        if ("N".equals(structure.getIsDictionary())){
            structure.setDictionaryTable("");
        }
        return this.saveOrUpdate(structure);
    }
}
