package com.muyu.etl.service.impl;

import java.util.List;
import java.util.concurrent.Future;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.utils.DateUtils;
import com.muyu.etl.domain.BasicConfigInfo;
import com.muyu.etl.domain.Structure;
import com.muyu.etl.domain.resp.TableInfoStructureResp;
import com.muyu.etl.service.BasicConfigInfoService;
import com.muyu.etl.service.StructureService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.muyu.etl.mapper.TableInfoMapper;
import com.muyu.etl.domain.TableInfo;
import com.muyu.etl.service.TableInfoService;

/**
 * 库表基础信息Service业务层处理
 *
 * @author Saisai
 * @date 2024-04-22
 */
@Log4j2
@Service
public class TableInfoServiceImpl extends ServiceImpl<TableInfoMapper, TableInfo> implements TableInfoService {
    @Autowired
    private TableInfoMapper tableInfoMapper;
    @Autowired
    private StructureService structureService;


    /**
     * 查询库表基础信息
     *
     * @param id 库表基础信息主键
     * @return 库表基础信息
     */
    @Override
    public TableInfo selectTableInfoById(Long id) {
        return tableInfoMapper.selectTableInfoById(id);
    }

    /**
     * 查询库表基础信息列表
     *
     * @param tableInfo 库表基础信息
     * @return 库表基础信息
     */
    @Override
    public List<TableInfo> selectTableInfoList(TableInfo tableInfo) {
        return tableInfoMapper.selectTableInfoList(tableInfo);
    }

    /**
     * 新增库表基础信息
     *
     * @param tableInfo 库表基础信息
     * @return 结果
     */
    @Override
    public int insertTableInfo(TableInfo tableInfo) {
        tableInfo.setCreateTime(DateUtils.getNowDate());
        return tableInfoMapper.insertTableInfo(tableInfo);
    }

    /**
     * 修改库表基础信息
     *
     * @param tableInfo 库表基础信息
     * @return 结果
     */
    @Override
    public int updateTableInfo(TableInfo tableInfo) {
        tableInfo.setUpdateTime(DateUtils.getNowDate());
        return tableInfoMapper.updateTableInfo(tableInfo);
    }

    /**
     * 批量删除库表基础信息
     *
     * @param ids 需要删除的库表基础信息主键
     * @return 结果
     */
    @Override
    public int deleteTableInfoByIds(Long[] ids) {
        return tableInfoMapper.deleteTableInfoByIds(ids);
    }

    /**
     * 删除库表基础信息信息
     *
     * @param id 库表基础信息主键
     * @return 结果
     */
    @Override
    public int deleteTableInfoById(Long id) {
        return tableInfoMapper.deleteTableInfoById(id);
    }

    @Override
    public TableInfo selectTableInfoByName(TableInfo table) {
        return tableInfoMapper.selectTableInfoByName(table);
    }


}
